/*=========================================================================*/
/*                        kafskonqplugin.cpp  -  description               */
/*                           -------------------                           */
/*  begin                : Mon Jan 13 21:48:25 CST 2003                    */
/*  copyright            : (C) 2003 by Ken Aaker                           */
/*  email                : kenaaker@myclearwave.net                        */
/*=========================================================================*/

/*=========================================================================*/
/*                                                                         */
/*   This program is free software; you can redistribute it and/or modify  */
/*   it under the terms of the GNU General Public License as published by  */
/*   the Free Software Foundation; either version 2 of the License, or     */
/*   (at your option) any later version.                                   */
/*                                                                         */
/*=========================================================================*/
/*=========================================================================*/

#include <kgenericfactory.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kapp.h>

#include <qwidgetstack.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <kcombobox.h>
#include <qbuttongroup.h>
#include <qerrormessage.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qdial.h>
#include <qcolor.h>

extern "C" {
#include <stdio.h>
#include <malloc.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <afs/stds.h>
#include <afs/afs.h>
#include <afs/cmd.h>
#include <afs/prs_fs.h>
#include <afs/vice.h>
#include <afs/venus.h>
#include <afs/afsint.h>
#include "human.h"
    extern long pioctl(const char *pathp, long opcode, struct ViceIoctl *blobp, int follow);
#define MAXSIZE 2048
    static char space[MAXSIZE];
} /* extern "C" */

#include "kafskonqplugin.h"
#include "afsperms.moc.h"

static void QuickPrintStatus(VolumeStatus *status, char *name) {
    double QuotaUsed =0.0;
    double PartUsed =0.0;
    int WARN = 0;
    printf("%-25.25s",name);

    if (status->MaxQuota != 0) {
	printf("%10d%10d", (int)status->MaxQuota, (int)status->BlocksInUse);
	QuotaUsed = ((((double)status->BlocksInUse)/status->MaxQuota) * 100.0);
    } else {
	printf("  no limit%10d", (int)status->BlocksInUse);
    }
    if (QuotaUsed > 90.0) {
	printf("%5.0f%%<<", QuotaUsed);
	WARN = 1;
    }
    else printf("%5.0f%%  ", QuotaUsed);
    PartUsed = (100.0 - ((((double)status->PartBlocksAvail)/status->PartMaxBlocks) * 100.0));
    if (PartUsed > 97.0) {
	printf("%9.0f%%<<", PartUsed);
	WARN = 1;
    }
    else printf("%9.0f%%  ", PartUsed);
    if (WARN){
	printf("  <<WARNING\n");
    }
    else printf("\n");
}
static void printRights(afs_int32 arights, int dfs) {
    if (!dfs) {
	if (arights & PRSFS_READ) { printf("r"); }
	if (arights & PRSFS_LOOKUP) { printf("l"); }
	if (arights & PRSFS_INSERT) { printf("i"); }
	if (arights & PRSFS_DELETE) { printf("d"); }
	if (arights & PRSFS_WRITE) { printf("w"); }
	if (arights & PRSFS_LOCK) { printf("k"); }
	if (arights & PRSFS_ADMINISTER) { printf("a"); }
	if (arights & PRSFS_USR0) { printf("A"); }
	if (arights & PRSFS_USR1) { printf("B"); }
	if (arights & PRSFS_USR2) { printf("C"); }
	if (arights & PRSFS_USR3) { printf("D"); }
	if (arights & PRSFS_USR4) { printf("E"); }
	if (arights & PRSFS_USR5) { printf("F"); }
	if (arights & PRSFS_USR6) { printf("G"); }
	if (arights & PRSFS_USR7) { printf("H"); }
    } else {
#ifdef BLAH
	if (arights & DFS_READ) printf("r"); else printf("-");
	if (arights & DFS_WRITE) printf("w"); else printf("-");
	if (arights & DFS_EXECUTE) printf("x"); else printf("-");
	if (arights & DFS_CONTROL) printf("c"); else printf("-");
	if (arights & DFS_INSERT) printf("i"); else printf("-");
	if (arights & DFS_DELETE) printf("d"); else printf("-");
	if (arights & (DFS_USRALL)) printf("+");
	if (arights & DFS_USR0) printf("A");
	if (arights & DFS_USR1) printf("B");
	if (arights & DFS_USR2) printf("C");
	if (arights & DFS_USR3) printf("D");
	if (arights & DFS_USR4) printf("E");
	if (arights & DFS_USR5) printf("F");
	if (arights & DFS_USR6) printf("G");
	if (arights & DFS_USR7) printf("H");
#endif
    }
}

/* this function returns TRUE (1) if the file is in AFS, otherwise false (0) */
static int inAFS(const char *apath) {
    struct ViceIoctl blob;
    afs_int32 code;
    afs_int32 rc;


    blob.in_size = 0;
    blob.out_size = sizeof(space);
    blob.out = space;

    code = pioctl((char *)apath, VIOC_FILE_CELL_NAME, &blob, 1);
    if (code == 0) {
	rc = 1;
    } else {
	if ((errno == EINVAL) || (errno == ENOENT)) {
	    rc = 0;
	} else {
	    rc = 1;
	}
    }
    return rc;
}

static char *skipLine(char *astr) {
    while (*astr !='\n') astr++;
    astr++;
    return astr;
}

static struct Acl *parseAcl(char *astr) {
    int nplus, nminus, i, trights;
    char tname[MAXNAME];
    struct AclEntry *first, *last, *tl;
    struct Acl *ta;

    ta = (struct Acl *) malloc (sizeof (struct Acl));
    ta->dfs = 0;
    sscanf(astr, "%d dfs:%d %s", &ta->nplus, &ta->dfs, ta->cell);
    astr = skipLine(astr);
    sscanf(astr, "%d", &ta->nminus);
    astr = skipLine(astr);

    nplus = ta->nplus;
    nminus = ta->nminus;

    last = 0;
    first = 0;
    for(i=0;i<nplus;i++) {
	sscanf(astr, "%100s %d", tname, &trights);
	astr = skipLine(astr);
	tl = (struct AclEntry *) malloc(sizeof (struct AclEntry));
	if (!first) first = tl;
	strcpy(tl->name, tname);
	tl->rights = trights;
	tl->next = 0;
	if (last) last->next = tl;
	last = tl;
    }
    ta->pluslist = first;

    last = 0;
    first = 0;
    for(i=0;i<nminus;i++) {
	sscanf(astr, "%100s %d", tname, &trights);
	astr = skipLine(astr);
	tl = (struct AclEntry *) malloc(sizeof (struct AclEntry));
	if (!first) first = tl;
	strcpy(tl->name, tname);
	tl->rights = trights;
	tl->next = 0;
	if (last) last->next = tl;
	last = tl;
    }
    ta->minuslist = first;

    return ta;
}

QString AclToString(struct Acl *acl)
{
    char mydata[MAXSIZE];
    char tstring[MAXSIZE];
    char dfsstring[30];
    struct AclEntry *tp;
    int plusEntries = 0;
    int minusEntries = 0;

    if (acl->dfs) {
	sprintf(dfsstring, " dfs:%d %s", acl->dfs, acl->cell);
    } else {
	dfsstring[0] = '\0';
    } // endif

    // Count the number of entries that have non-0 rights.
    for (tp = acl->pluslist; tp; tp = tp->next) {
	if (tp->rights > 0) {
	    ++plusEntries;
	} /* endif */
    } // endfor

    for (tp = acl->minuslist; tp; tp = tp->next) {
	if (tp->rights > 0) {
	    ++minusEntries;
	} /* endif */
    } // endfor

    sprintf(mydata, "%d%s\n%d\n", plusEntries, dfsstring, minusEntries);

    for (tp = acl->pluslist; tp; tp = tp->next) {
	if (tp->rights > 0) {
	    sprintf(tstring, "%s %ld\n", tp->name, tp->rights);
	    strcat(mydata, tstring);
	} /* endif */
    } // endfor

    for (tp = acl->minuslist; tp; tp = tp->next) {
	if (tp->rights > 0) {
	    sprintf(tstring, "%s %ld\n", tp->name, tp->rights);
	    strcat(mydata, tstring);
	} /* endif */
    } // endfor

    return QString(mydata);
}


static void setNormRights(afs_int32 arights, AFSPermsPage *thePage) {
    thePage->NormRead->setChecked(arights & PRSFS_READ);
    thePage->NormLookup->setChecked(arights & PRSFS_LOOKUP);
    thePage->NormInsert->setChecked(arights & PRSFS_INSERT);
    thePage->NormDelete->setChecked(arights & PRSFS_DELETE);
    thePage->NormWrite->setChecked(arights & PRSFS_WRITE);
    thePage->NormLock->setChecked(arights & PRSFS_LOCK);
    thePage->NormAdmin->setChecked(arights & PRSFS_ADMINISTER);
    thePage->NormUserA->setChecked(arights & PRSFS_USR0);
    thePage->NormUserB->setChecked(arights & PRSFS_USR1);
    thePage->NormUserC->setChecked(arights & PRSFS_USR2);
    thePage->NormUserD->setChecked(arights & PRSFS_USR3);
    thePage->NormUserE->setChecked(arights & PRSFS_USR4);
    thePage->NormUserF->setChecked(arights & PRSFS_USR5);
    thePage->NormUserG->setChecked(arights & PRSFS_USR6);
    thePage->NormUserH->setChecked(arights & PRSFS_USR7);
}

static void setNegRights(afs_int32 arights, AFSPermsPage *thePage) {
    thePage->NegRead->setChecked(arights & PRSFS_READ);
    thePage->NegLookup->setChecked(arights & PRSFS_LOOKUP);
    thePage->NegInsert->setChecked(arights & PRSFS_INSERT);
    thePage->NegDelete->setChecked(arights & PRSFS_DELETE);
    thePage->NegWrite->setChecked(arights & PRSFS_WRITE);
    thePage->NegLock->setChecked(arights & PRSFS_LOCK);
    thePage->NegAdmin->setChecked(arights & PRSFS_ADMINISTER);
    thePage->NegUserA->setChecked(arights & PRSFS_USR0);
    thePage->NegUserB->setChecked(arights & PRSFS_USR1);
    thePage->NegUserB->setChecked(arights & PRSFS_USR2);
    thePage->NegUserB->setChecked(arights & PRSFS_USR3);
    thePage->NegUserB->setChecked(arights & PRSFS_USR4);
    thePage->NegUserB->setChecked(arights & PRSFS_USR5);
    thePage->NegUserB->setChecked(arights & PRSFS_USR6);
    thePage->NegUserB->setChecked(arights & PRSFS_USR7);
}

static struct Acl *getACL(QString &path) {
    int rc;
    struct Acl *ta = NULL;
    struct ViceIoctl blob;
#ifdef KAFS_DEBUG
    struct AclEntry *te;
#endif
    int error = 0;
    blob.out_size = MAXSIZE;
    blob.in_size = 1;
    blob.in = blob.out = space;
    rc = pioctl(path.latin1(), VIOCGETAL, &blob, 1);
    if (rc) {
	error = 1;
    } else {
	ta = parseAcl(space);
#ifdef KAFS_DEBUG
	switch (ta->dfs) {
	    case 0:
		printf("Access list for %s is\n", path.latin1());
		break;
	    case 1:
		printf("DFS access list for %s is\n", path.latin1());
		break;
	    case 2:
		printf("DFS initial directory access list of %s is\n", path.latin1());
		break;
	    case 3:
		printf("DFS initial file access list of %s is\n", path.latin1());
		break;
	}
	if (ta->dfs) {
	    printf("  Default cell = %s\n", ta->cell);
	}
	if (ta->nplus > 0) {
	    if (!ta->dfs) {
		printf("Normal rights:\n");
	    }
	    for(te = ta->pluslist; te; te=te->next) {
		printf("  %s ", te->name);
		printRights(te->rights, ta->dfs);
		printf("\n");
	    }
	}
	if (ta->nminus > 0) {
	    printf("Negative rights:\n");
	    for(te = ta->minuslist; te; te=te->next) {
		printf("  %s ", te->name);
		printRights(te->rights, ta->dfs);
		printf("\n");
	    }
	}
#endif
    }
    return ta;
}

typedef KGenericFactory<KAFSPropertiesDialogPlugin, KPropertiesDialog> KAFSPropertiesDialogPluginFactory;

K_EXPORT_COMPONENT_FACTORY(kafskonqplugin,
			   KAFSPropertiesDialogPluginFactory("kafskonqplugin"));

void KAFSPropertiesDialogPlugin::getQuota(QString &path) {
    afs_int32 code;
    struct ViceIoctl blob;

    struct {
	struct VolumeStatus status;       // Volume Status area
	char volName[MAXNAME] __attribute__((packed));// Undocumented, but this is how the fs.c code works.
    } statusName;

#ifdef KAFS_DEBUG
    printf("%-25s%-10s%-10s%-7s%-11s\n",
	   "Volume Name","     Quota", "      Used", " %Used", "  Partition");
#endif
    blob.out_size = sizeof(statusName);
    blob.in_size = 0;
    blob.out = (char *)&statusName;
    code = pioctl(path.latin1(), VIOCGETVOLSTAT, &blob, 1);
    if (code == 0) {
#ifdef KAFS_DEBUG
	QuickPrintStatus(&statusName.status, &statusName.volName[0]);
#endif
	afsVolumeName = statusName.volName;
	afsVolumeID = statusName.status.Vid;
	maxQuota = statusName.status.MaxQuota;
	minQuota = statusName.status.MinQuota;
	partitionAvailBlocks = statusName.status.PartBlocksAvail;
	partitionBlocks = statusName.status.PartMaxBlocks;
	blocksInUse = statusName.status.BlocksInUse;
    } else {
	afsVolumeName = "";
	afsVolumeID = 0;
	maxQuota = 0;
	blocksInUse = 0;
    } /* endif */
    return;
}

void KAFSPropertiesDialogPlugin::initValues(AFSPermsPage *thePage) {
    // Given the name associated with the  permissions, set the checkboxes for this user.
    int i;
    int percentNumber;
    int partPercentNumber;
    char *retString;
    QString partPercentage;
    long long availableBytes;
    QString volPercentage;
     char rawNumber[128];
   QString rawNumberQString;
    struct AclEntry *theAclEntry;

    thePage->DirectoryName->setText(dirPath);
    if (thisAcl) {
	// Set up to capture the return Keys on the ComboBoxes.
	thePage->NormRightsUsersCB->setTrapReturnKey(TRUE);
	thePage->NegRightsUsersCB->setTrapReturnKey(TRUE);
	// Initialize the combobox names.
	for (theAclEntry = thisAcl->pluslist; theAclEntry != NULL;
	theAclEntry = theAclEntry->next) {
	    thePage->NormRightsUsersCB->insertItem(tr2i18n(theAclEntry->name));
	} /* endfor */
	for (theAclEntry = thisAcl->minuslist; theAclEntry != NULL;
	theAclEntry = theAclEntry->next) {
	    thePage->NegRightsUsersCB->insertItem(tr2i18n(theAclEntry->name));
	} /* endfor */

	thePage->NormRightsUsersCB->setCurrentItem(0);
	if (thisAcl->nplus > 0) {
	    setNormRights(thisAcl->pluslist->rights, thePage);
	} /* endif */
	if (thisAcl->nminus > 0) {
	    setNegRights(thisAcl->minuslist->rights, thePage);
	} /* endif */
	thePage->AFSVolumeName->setText(tr2i18n(afsVolumeName));
	thePage->VolumeID->setText(tr2i18n(QString::number(afsVolumeID)));
	availableBytes = (long long)maxQuota * 1024LL;
#ifdef BLAH
	snprintf(rawNumber, sizeof(rawNumber), "%lld", availableBytes);
	rawNumberQString = rawNumber;
	for (i = (rawNumberQString.length()-3); i > 0; i -= 3) {
	    rawNumberQString.insert(i, ',');
	} /* endfor */
	thePage->QuotaSize->setText(rawNumberQString);
#else
	retString = human_readable(availableBytes, &rawNumber[0],
				   (human_group_digits | human_autoscale |
				    human_base_1024 | human_SI | human_B),
				   1024, 1024);

	rawNumberQString = retString;
	thePage->QuotaSize->setText(rawNumberQString);
	retString = human_readable(partitionAvailBlocks, &rawNumber[0],
				   (human_group_digits | human_autoscale |
				    human_base_1024 | human_SI | human_B),
				   1024, 1);

	rawNumberQString = retString;
	thePage->PartitionFreeSpace->setText(rawNumberQString);
#endif
	percentNumber = (blocksInUse * 100)/(maxQuota - minQuota);

	volPercentage = QString::number(percentNumber);
	volPercentage += '%';
	thePage->VolUtilization->setText(tr2i18n(volPercentage));

	thePage->QuotaDial->setValue(percentNumber);
	if ((percentNumber > 0) && (percentNumber <= 75)) {
	    thePage->QuotaDial->setPaletteBackgroundColor(Qt::green);
	} else {
	    if ((percentNumber > 75) && (percentNumber <= 90)) {
		thePage->QuotaDial->setPaletteBackgroundColor(Qt::yellow);
	    } else {
		if ((percentNumber > 90) && (percentNumber <= 99)) {
		    thePage->QuotaDial->setPaletteBackgroundColor(Qt::red);
		} else {
		    thePage->QuotaDial->setPaletteBackgroundColor(Qt::white);
		} /* endif */
	    } /* endif */
	} /* endif */

	partPercentNumber = (int)(((long long)(partitionBlocks - partitionAvailBlocks) * 100LL)/(long long)partitionBlocks);
	partPercentage = QString::number(partPercentNumber);

	partPercentage += '%';
//        printf("%s:%s:%d partPercentage 3 = %s.\n", __FILE__, __FUNCTION__, __LINE__, partPercentage.ascii());

	thePage->PartUtilization->setText(tr2i18n(partPercentage));
	thePage->PartitionDial->setValue(partPercentNumber);

	if ((partPercentNumber > 0) && (partPercentNumber <= 75)) {
	    thePage->PartitionDial->setPaletteBackgroundColor(Qt::green);
	} else {
	    if ((partPercentNumber > 75) && (partPercentNumber <= 90)) {
		thePage->PartitionDial->setPaletteBackgroundColor(Qt::yellow);
	    } else {
		if ((partPercentNumber > 90) && (partPercentNumber <= 99)) {
		    thePage->PartitionDial->setPaletteBackgroundColor(Qt::red);
		} else {
		    thePage->PartitionDial->setPaletteBackgroundColor(Qt::white);
		} /* endif */
	    } /* endif */
	} /* endif */

    } else {
    }
}

void KAFSPropertiesDialogPlugin::setUserPerms(const QString & ptsName,
                                              const int aclListType,
					      const int permissionID,
					      const int newValue)
{
    // Given the name associated with the  permissions, set the ACL structure for this user.
    struct AclEntry *startList;
    struct AclEntry *theAclEntry;
    struct AclEntry *foundAclEntry;
    afs_int32 rc;
    struct ViceIoctl blob;
    unsigned long savedRights;

#ifdef KAFS_DEBUG
    printf("%s:%s:%d Enter newValue = %d\n", __FILE__, __FUNCTION__, __LINE__, newValue);
    printf("User %s passed in.\n", ptsName.latin1());
#endif
    // Find the user name in the ACL list, or add it.
    foundAclEntry = NULL;
    if (aclListType >= 0) {
        startList = thisAcl->pluslist;
    } else {
        startList = thisAcl->minuslist;
    } /* endif */
    for (theAclEntry = startList; theAclEntry != NULL; theAclEntry = theAclEntry->next) {
	if (tr2i18n(theAclEntry->name) == ptsName) {
	    foundAclEntry = theAclEntry;
	    break;
	} /* endif */
    } /* endfor */

    if (foundAclEntry == NULL) {
	struct AclEntry *tl;
#ifdef KAFS_DEBUG
	printf("Didn't find user %s, creating new ACL entry.\n", ptsName.latin1());
#endif
	tl = (struct AclEntry *) malloc(sizeof (struct AclEntry));
	tl->next = startList;
	strcpy(tl->name, ptsName.latin1());
	tl->rights = 0;
        savedRights = 0;
        if (aclListType >= 0) {
            setNormRights(tl->rights, AFSPermWidget);
            thisAcl->pluslist = tl;
            ++thisAcl->nplus;
        } else {
            setNegRights(tl->rights, AFSPermWidget);
            thisAcl->minuslist = tl;
            ++thisAcl->nminus;
        } /* endif */
	foundAclEntry = tl;
    } /* endif */

#ifdef KAFS_DEBUG
    printf("Working with foundAclEntry->name = %s.\n", foundAclEntry->name);
#endif
    if (tr2i18n(foundAclEntry->name) == ptsName) {
	savedRights = foundAclEntry->rights;
	if (newValue) {
	    foundAclEntry->rights |= permissionID;
	} else {
	    foundAclEntry->rights &= ~permissionID;
	} /* endif */
	strncpy(space, AclToString(thisAcl), MAXSIZE);
	blob.in = space;
#ifdef KAFS_DEBUG
	printf("Ready for pioctl with ACL string %s.\n", blob.in);
#endif
	blob.out_size = 0;
	blob.in_size = 1 + strlen(blob.in);
	rc = pioctl(dirPath.latin1(), VIOCSETAL, &blob, 1);
	if (rc != 0) {
	    // Put the rights back the way they were.
	    foundAclEntry->rights = savedRights;
	    if (errno == EINVAL) {
              QMessageBox::warning(propDialog,
                                   tr("AFS_ACL_Message"),
                                   tr(" Invalid argument, possible reasons include:\n"
                                      "\t-Too many users on access control list\n"
                                      "\t-Tried to add non-existent user to access control list"
                                      ));
            } else {
                QString errNumber;
                errNumber.setNum(errno);
                QMessageBox::warning(propDialog,
                                     tr("AFS_ACL_Message"),
                                     tr("Unknown error errno = ") +
                                     tr(errNumber));
	    } /* endif */
	} /* endif */
        if (aclListType >= 0) {
            setNormRights(foundAclEntry->rights, AFSPermWidget);
        } else {
            setNegRights(foundAclEntry->rights, AFSPermWidget);
        } /* endif */
    } /* endif */
#ifdef KAFS_DEBUG
    printf("%s:%s:%d Exit\n", __FILE__, __FUNCTION__, __LINE__);
#endif
}

void KAFSPropertiesDialogPlugin::displayUserPerms(const QString & ptsName, const int aclListType)
{
  // Given the name associated with the  permissions, set the checkboxes for this user.
  struct AclEntry *theAclEntry;
  unsigned long rights = 0;

  if (aclListType >= 0) {
      theAclEntry = thisAcl->pluslist;
  } else {
      theAclEntry = thisAcl->minuslist;
  } /* endif */

  for (; theAclEntry != NULL;
       theAclEntry = theAclEntry->next) {
    if (tr2i18n(theAclEntry->name) == ptsName) {
#ifdef KAFS_DEBUG
      printf("User  %s selected\n", ptsName.latin1());
#endif
      rights = theAclEntry->rights;
      break;
    } /* endif */
  } /* endfor */
  if (aclListType >= 0) {
      setNormRights(rights, AFSPermWidget);
  } else {
      setNegRights(rights, AFSPermWidget);
  } /* endif */
}

#define MAX_PERMS 15
static const int permIDValues[MAX_PERMS] = {
    /*  0 */	PRSFS_LOOKUP,
    /*  1 */	PRSFS_READ,
    /*  2 */	PRSFS_WRITE,
    /*  3 */	PRSFS_ADMINISTER,
    /*  4 */	PRSFS_INSERT,
    /*  5 */	PRSFS_DELETE,
    /*  6 */	PRSFS_LOCK,
    /*  7 */	PRSFS_USR0,
    /*  8 */	PRSFS_USR1,
    /*  9 */	PRSFS_USR2,
    /* 10 */	PRSFS_USR3,
    /* 11 */	PRSFS_USR4,
    /* 12 */	PRSFS_USR5,
    /* 13 */	PRSFS_USR6,
    /* 14 */	PRSFS_USR7 
};

void KAFSPropertiesDialogPlugin::normPermChanged(int permButtonID)
{
    if ((permButtonID < 0) || (permButtonID >= MAX_PERMS)) {
	printf("%s:%s:%d Permission ID value is out of range (%d), ignoring.\n", __FILE__, __FUNCTION__, __LINE__, permButtonID);
    } else {
	int permBit = permIDValues[permButtonID];
	int newValue = ((QCheckBox *)AFSPermWidget->NormRightsBG->find(permButtonID))->state();
#ifdef KAFS_DEBUG
	printf("%s:%s:%d Permission ID value is %d.\n", __FILE__, __FUNCTION__, __LINE__, permButtonID);
#endif
	setUserPerms(AFSPermWidget->NormRightsUsersCB->currentText(), 1, permBit, newValue);
	displayUserPerms(AFSPermWidget->NormRightsUsersCB->currentText(), 1);
    } /* endif */
}

void KAFSPropertiesDialogPlugin::negPermChanged(int permButtonID)
{
    if ((permButtonID < 0) || (permButtonID >= MAX_PERMS)) {
	printf("%s:%s:%d Permission ID value is out of range (%d), ignoring\n", __FILE__, __FUNCTION__, __LINE__, permButtonID);
    } else {
	int permBit = permIDValues[permButtonID];
	int newValue = ((QCheckBox *)AFSPermWidget->NegRightsBG->find(permButtonID))->state();
	setUserPerms(AFSPermWidget->NegRightsUsersCB->currentText(), -1, permBit, newValue);
	displayUserPerms(AFSPermWidget->NegRightsUsersCB->currentText(), -1);
    } /* endif */
}


AFSPermsPage* KAFSPropertiesDialogPlugin::createAFSPermWidget(QWidget* parent) {
    AFSPermsPage *thePage = NULL;

    if (!dirPath.isEmpty()){
	thisAcl = getACL(dirPath);
	getQuota(dirPath);
	thePage = new AFSPermsPage(parent);
	initValues(thePage);
    }
    //  connect(thePage->permissionsBox, SIGNAL(clicked(int)), this, SLOT(slotSharedChanged(int)));
    connect(thePage, SIGNAL(UserChanged(const QString &, const int)),
	    this, SLOT(displayUserPerms(const QString &, const int)));
    connect(thePage->NormRightsBG, SIGNAL(clicked(int)), this, SLOT(normPermChanged(int)));
    connect(thePage->NegRightsBG, SIGNAL(clicked(int)), this, SLOT(negPermChanged(int)));
    return thePage;
}

KAFSPropertiesDialogPlugin::KAFSPropertiesDialogPlugin(KPropertiesDialog *dlg,
						       const char *,
						       const QStringList &) : KPropsDlgPlugin(dlg) {
    propDialog = dlg;

    // Only accept AFS files
    thisAcl = NULL;
    if (propDialog->kurl().isLocalFile()) {
	if (inAFS(propDialog->kurl().path().latin1())) {
	    dirPath = propDialog->kurl().path();
	    page = propDialog->addPage(i18n("AFS"));

	    AFSPermWidget = createAFSPermWidget((QWidget *)page);
	    page->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, 0, 0, 0));
	    page->setMinimumSize(AFSPermWidget->size());
	} /* endif */
    } /* endif */
}

KAFSPropertiesDialogPlugin::~KAFSPropertiesDialogPlugin()
{
    struct AclEntry *theAclEntry;
    struct AclEntry *nextEntry;

    printf("%s: Ummmm---Destruction\n", __FUNCTION__); 
    if (thisAcl) {
	// Initialize the combobox names.
	for (theAclEntry = thisAcl->pluslist; theAclEntry != NULL;
	theAclEntry = nextEntry) {
	    nextEntry = theAclEntry->next;
	    free(theAclEntry);
	} /* endfor */
	for (theAclEntry = thisAcl->minuslist; theAclEntry != NULL;
	theAclEntry = nextEntry) {
	    nextEntry = theAclEntry->next;
	    free(theAclEntry);
	} /* endfor */
	free(thisAcl);
	thisAcl = NULL;
    } /* endif */
} /* KAFSPropertiesDialogPlugin:: ~ */


bool KAFSPropertiesDialogPlugin::checkValues()
{
    return true;
}

void KAFSPropertiesDialogPlugin::applyChanges()
{
    return;
}


